@@ -1,67 +1,12 @@
-#' Add attributes to an object
-#'
-#' `set_attrs()` adds, changes, or zaps attributes of objects. Pass a
-#' single unnamed `NULL` as argument to zap all attributes. For
-#' [uncopyable][is_copyable] types, use `mut_attrs()`.
-#'
-#' Unlike [structure()], these setters have no special handling of
-#' internal attributes names like `.Dim`, `.Dimnames` or `.Names`.
-#'
-#' @param .x An object to decorate with attributes.
-#' @param ... A list of named attributes. These have [explicit
-#'   splicing semantics][dots_list]. Pass a single unnamed `NULL` to
-#'   zap all attributes from `.x`.
-#' @return `set_attrs()` returns a modified [shallow copy][duplicate]
-#'   of `.x`. `mut_attrs()` invisibly returns the original `.x`
-#'   modified in place.
-#' @export
-#' @examples
-#' set_attrs(letters, names = 1:26, class = "my_chr")
-#'
-#' # Splice a list of attributes:
-#' attrs <- list(attr = "attr", names = 1:26, class = "my_chr")
-#' obj <- set_attrs(letters, splice(attrs))
-#' obj
-#'
-#' # Zap attributes by passing a single unnamed NULL argument:
-#' set_attrs(obj, NULL)
-#' set_attrs(obj, !!! list(NULL))
-#'
-#' # Note that set_attrs() never modifies objects in place:
-#' obj
-#'
-#' # For uncopyable types, mut_attrs() lets you modify in place:
-#' env <- env()
-#' mut_attrs(env, foo = "bar")
-#' env
-set_attrs <- function(.x, ...) {
-  if (!is_copyable(.x)) {
-    abort("`.x` is uncopyable: use `mut_attrs()` to change attributes in place")
-  }
-  set_attrs_impl(.x, ...)
-}
-#' @rdname set_attrs
-#' @export
-mut_attrs <- function(.x, ...) {
-  if (is_copyable(.x)) {
-    abort("`.x` is copyable: use `set_attrs()` to change attributes without side effect")
-  }
-  invisible(set_attrs_impl(.x, ...))
-}
-set_attrs_impl <- function(.x, ...) {
-  attrs <- dots_list(...)
 
-  # If passed a single unnamed NULL, zap attributes
-  if (identical(attrs, set_attrs_null)) {
-    attributes(.x) <- NULL
-  } else {
-    attributes(.x) <- c(attributes(.x), attrs)
-  }
+structure2 <- function(.x, ...) {
+  exec("structure", .Data = .x, ...)
+}
 
-  .x
+set_class <- function(x, class) {
+  attr(x, "class") <- class
+  x
 }
-set_attrs_null <- list(NULL)
-names(set_attrs_null) <- ""
 
 #' Is object named?
 #'
@@ -121,7 +66,7 @@ is_named <- function(x) {
     return(FALSE)
   }
 
-  if (any(nms == "" | is.na(nms))) {
+  if (any(nms_are_invalid(nms))) {
     return(FALSE)
   }
 
@@ -143,10 +88,14 @@ have_name <- function(x) {
   if (is.null(nms)) {
     rep(FALSE, length(x))
   } else {
-    !(is.na(nms) | nms == "")
+    !nms_are_invalid(nms)
   }
 }
 
+nms_are_invalid <- function(x) {
+  x == "" | is.na(x)
+}
+
 #' Does an object have an element with this name?
 #'
 #' This function returns a logical value that indicates if a data frame or
@@ -168,9 +117,18 @@ has_name <- function(x, name) {
 
 #' Set names of a vector
 #'
+#' @description
+#'
+#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("stable")}
+#'
 #' This is equivalent to [stats::setNames()], with more features and
 #' stricter argument checking.
 #'
+#'
+#' @section Life cycle:
+#'
+#' `set_names()` is stable and exported in purrr.
+#'
 #' @param x Vector to name.
 #' @param nm,... Vector of names, the same length as `x`.
 #'
@@ -184,9 +142,8 @@ has_name <- function(x, name) {
 #'
 #'   * If `nm` is `NULL`, the names are removed (if present).
 #'
-#'   * In all other cases, `nm` and `...` are passed to [chr()]. This
-#'     gives implicit splicing semantics: you can pass character
-#'     vectors or list of character vectors indistinctly.
+#'   * In all other cases, `nm` and `...` are coerced to character.
+#'
 #' @export
 #' @examples
 #' set_names(1:4, c("a", "b", "c", "d"))
@@ -203,17 +160,22 @@ has_name <- function(x, name) {
 #' # `...` is passed to the function:
 #' set_names(head(mtcars), paste0, "_foo")
 set_names <- function(x, nm = x, ...) {
+  set_names_impl(x, x, nm, ...)
+}
+set_names_impl <- function(x, mold, nm, ...) {
   if (!is_vector(x)) {
     abort("`x` must be a vector")
   }
 
   if (is_function(nm) || is_formula(nm)) {
     nm <- as_function(nm)
-    nm <- nm(names2(x), ...)
+    nm <- nm(names2(mold), ...)
   } else if (!is_null(nm)) {
-    # Make sure `x` is serialised when no arguments is provided.
-    nm <- as.character(nm)
-    nm <- chr(nm, ...)
+    if (dots_n(...)) {
+      nm <- as.character(c(nm, ...))
+    } else {
+      nm <- as.character(nm)
+    }
   }
 
   if (!is_null(nm) && !is_character(nm, length(x))) {
@@ -226,11 +188,20 @@ set_names <- function(x, nm = x, ...) {
 
 #' Get names of a vector
 #'
+#' @description
+#'
+#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("stable")}
+#'
 #' This names getter always returns a character vector, even when an
 #' object does not have a `names` attribute. In this case, it returns
 #' a vector of empty names `""`. It also standardises missing names to
 #' `""`.
 #'
+#'
+#' @section Life cycle:
+#'
+#' `names2()` is stable.
+#'
 #' @param x A vector.
 #' @export
 #' @examples
@@ -280,3 +251,7 @@ has_length <- function(x, n = NULL) {
     len == n
   }
 }
+
+poke_attributes <- function(x, attrs) {
+  .Call(rlang_poke_attributes, x, attrs)
+}
