@@ -12,15 +12,16 @@
 #' * Unlike `is.atomic()`, `is_atomic()` does not return `TRUE` for
 #'   `NULL`.
 #'
-#' * Unlike `is.vector()`, `is_vector()` test if an object is an
+#' * Unlike `is.vector()`, `is_vector()` tests if an object is an
 #'   atomic vector or a list. `is.vector` checks for the presence of
 #'   attributes (other than name).
 #'
-#' * `is_function()` returns `TRUE` only for regular functions, not
-#'   special or primitive functions.
-#'
 #' @param x Object to be tested.
 #' @param n Expected length of a vector.
+#' @param finite Whether all values of the vector are finite. The
+#'   non-finite values are `NA`, `Inf`, `-Inf` and `NaN`. Setting this
+#'   to something other than `NULL` can be expensive because the whole
+#'   vector needs to be traversed and checked.
 #' @param encoding Expected encoding of a string or character
 #'   vector. One of `UTF-8`, `latin1`, or `unknown`.
 #' @seealso [bare-type-predicates] [scalar-type-predicates]
@@ -30,9 +31,7 @@ NULL
 #' @export
 #' @rdname type-predicates
 is_list <- function(x, n = NULL) {
-  if (typeof(x) != "list") return(FALSE)
-  if (!is_null(n) && length(x) != n) return(FALSE)
-  TRUE
+  .Call(rlang_is_list, x, n)
 }
 
 parsable_atomic_types <- c("logical", "integer", "double", "complex", "character")
@@ -40,52 +39,55 @@ atomic_types <- c(parsable_atomic_types, "raw")
 #' @export
 #' @rdname type-predicates
 is_atomic <- function(x, n = NULL) {
-  if (!typeof(x) %in% atomic_types) return(FALSE)
-  if (!is_null(n) && length(x) != n) return(FALSE)
-  TRUE
+  .Call(rlang_is_atomic, x, n)
 }
 #' @export
 #' @rdname type-predicates
 is_vector <- function(x, n = NULL) {
-  is_atomic(x, n) || is_list(x, n)
+  .Call(rlang_is_vector, x, n)
+}
+
+# Mostly for unit testing
+is_finite <- function(x) {
+  .Call(rlang_is_finite, x)
 }
 
 #' @export
 #' @rdname type-predicates
 is_integer <- function(x, n = NULL) {
-  if (typeof(x) != "integer") return(FALSE)
-  if (!is_null(n) && length(x) != n) return(FALSE)
-  TRUE
+  .Call(rlang_is_integer, x, n)
 }
 #' @export
 #' @rdname type-predicates
-is_double <- function(x, n = NULL) {
-  if (typeof(x) != "double") return(FALSE)
-  if (!is_null(n) && length(x) != n) return(FALSE)
-  TRUE
+is_double <- function(x, n = NULL, finite = NULL) {
+  .Call(rlang_is_double, x, n, finite)
 }
 #' @export
 #' @rdname type-predicates
 is_character <- function(x, n = NULL, encoding = NULL) {
-  if (typeof(x) != "character") return(FALSE)
-  if (!is_null(n) && length(x) != n) return(FALSE)
-  stopifnot(typeof(encoding) %in% c("character", "NULL"))
-  if (!is_null(encoding) && !all(chr_encoding(x) %in% encoding)) return(FALSE)
+  if (!is_null(encoding)) {
+    warn_deprecated("The `encoding` argument is deprecated as of rlang 0.3.0.")
+  }
+  if (!.Call(rlang_is_character, x, n)) {
+    return(FALSE)
+  }
+  if (!is_null(encoding)) {
+    stopifnot(typeof(encoding) == "character")
+    if (!all(chr_encoding(x) %in% encoding)) {
+      return(FALSE)
+    }
+  }
   TRUE
 }
 #' @export
 #' @rdname type-predicates
 is_logical <- function(x, n = NULL) {
-  if (typeof(x) != "logical") return(FALSE)
-  if (!is_null(n) && length(x) != n) return(FALSE)
-  TRUE
+  .Call(rlang_is_logical, x, n)
 }
 #' @export
 #' @rdname type-predicates
 is_raw <- function(x, n = NULL) {
-  if (typeof(x) != "raw") return(FALSE)
-  if (!is_null(n) && length(x) != n) return(FALSE)
-  TRUE
+  .Call(rlang_is_raw, x, n)
 }
 #' @export
 #' @rdname type-predicates
@@ -94,7 +96,7 @@ is_bytes <- is_raw
 #' @export
 #' @rdname type-predicates
 is_null <- function(x) {
-  typeof(x) == "NULL"
+  .Call(rlang_is_null, x)
 }
 
 #' Scalar type predicates
@@ -110,46 +112,54 @@ NULL
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_list <- function(x) {
-  is_list(x) && length(x) == 1
+  .Call(rlang_is_list, x, 1L)
 }
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_atomic <- function(x) {
-  is_atomic(x) && length(x) == 1
+  .Call(rlang_is_atomic, x, 1L)
 }
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_vector <- function(x) {
-  is_vector(x) && length(x) == 1
+  .Call(rlang_is_vector, x, 1L)
 }
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_integer <- function(x) {
-  is_integer(x) && length(x) == 1
+  .Call(rlang_is_integer, x, 1L)
 }
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_double <- function(x) {
-  is_double(x) && length(x) == 1
+  .Call(rlang_is_double, x, 1L, NULL)
 }
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_character <- function(x, encoding = NULL) {
-  is_character(x, encoding = encoding) && length(x) == 1
+  if (!is_null(encoding)) {
+    warn_deprecated("The `encoding` argument is deprecated as of rlang 0.3.0.")
+  }
+  is_character(x, encoding = encoding, n = 1L)
 }
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_logical <- function(x) {
-  is_logical(x) && length(x) == 1
+  .Call(rlang_is_logical, x, 1L)
 }
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_raw <- function(x) {
-  is_raw(x) && length(x) == 1
+  .Call(rlang_is_raw, x, 1L)
 }
 #' @export
 #' @rdname scalar-type-predicates
-is_string <- is_scalar_character
+is_string <- function(x, encoding = NULL) {
+  if (!is_null(encoding)) {
+    warn_deprecated("The `encoding` argument is deprecated as of rlang 0.3.0.")
+  }
+  is_character(x, encoding = encoding, n = 1L) && !is.na(x)
+}
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_bytes <- is_scalar_raw
@@ -207,6 +217,9 @@ is_bare_numeric <- function(x, n = NULL) {
 #' @export
 #' @rdname bare-type-predicates
 is_bare_character <- function(x, n = NULL, encoding = NULL) {
+  if (!is_null(encoding)) {
+    warn_deprecated("The `encoding` argument is deprecated as of rlang 0.3.0.")
+  }
   !is.object(x) && is_character(x, n, encoding = encoding)
 }
 #' @export
@@ -241,17 +254,17 @@ is_empty <- function(x) length(x) == 0
 
 #' Is object an environment?
 #'
-#' `is_bare_env()` tests whether `x` is an environment without a s3 or
+#' `is_bare_environment()` tests whether `x` is an environment without a s3 or
 #' s4 class.
 #'
 #' @inheritParams is_empty
 #' @export
-is_env <- function(x) {
+is_environment <- function(x) {
   typeof(x) == "environment"
 }
-#' @rdname is_env
+#' @rdname is_environment
 #' @export
-is_bare_env <- function(x) {
+is_bare_environment <- function(x) {
   !is.object(x) && typeof(x) == "environment"
 }
 
@@ -278,12 +291,24 @@ is_false <- function(x) {
 
 #' Is a vector integer-like?
 #'
+#' @description
+#'
 #' These predicates check whether R considers a number vector to be
 #' integer-like, according to its own tolerance check (which is in
 #' fact delegated to the C library). This function is not adapted to
 #' data analysis, see the help for [base::is.integer()] for examples
 #' of how to check for whole numbers.
 #'
+#' Things to consider when checking for integer-like doubles:
+#'
+#' * This check can be expensive because the whole double vector has
+#'   to be traversed and checked.
+#'
+#' * Large double values may be integerish but may still not be
+#'   coercible to integer. This is because integers in R only support
+#'   values up to `2^31 - 1` while numbers stored as double can be
+#'   much larger.
+#'
 #' @seealso [is_bare_numeric()] for testing whether an object is a
 #'   base numeric type (a bare double or integer vector).
 #' @inheritParams type-predicates
@@ -291,35 +316,44 @@ is_false <- function(x) {
 #' @examples
 #' is_integerish(10L)
 #' is_integerish(10.0)
+#' is_integerish(10.0, n = 2)
 #' is_integerish(10.000001)
 #' is_integerish(TRUE)
-is_integerish <- function(x, n = NULL) {
-  if (typeof(x) == "integer") return(TRUE)
-  if (typeof(x) != "double") return(FALSE)
-  if (!is_null(n) && length(x) != n) return(FALSE)
-  all(x == as.integer(x))
+is_integerish <- function(x, n = NULL, finite = NULL) {
+  .Call(rlang_is_integerish, x, n, finite)
 }
 #' @rdname is_integerish
 #' @export
-is_bare_integerish <- function(x, n = NULL) {
-  !is.object(x) && is_integerish(x, n)
+is_bare_integerish <- function(x, n = NULL, finite = NULL) {
+  !is.object(x) && is_integerish(x, n, finite)
 }
 #' @rdname is_integerish
 #' @export
-is_scalar_integerish <- function(x) {
-  !is.object(x) && is_integerish(x, 1L)
+is_scalar_integerish <- function(x, finite = NULL) {
+  .Call(rlang_is_integerish, x, 1L, finite)
 }
 
 #' Base type of an object
 #'
+#' @description
+#'
+#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("experimental")}
+#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("questioning")}
+#'
 #' This is equivalent to [base::typeof()] with a few differences that
 #' make dispatching easier:
 #' * The type of one-sided formulas is "quote".
 #' * The type of character vectors of length 1 is "string".
 #' * The type of special and builtin functions is "primitive".
 #'
+#'
+#' @section Life cycle:
+#'
+#' `type_of()` is an experimental function. Expect API changes.
+#'
 #' @param x An R object.
 #' @export
+#' @keywords internal
 #' @examples
 #' type_of(10L)
 #'
@@ -350,7 +384,7 @@ is_scalar_integerish <- function(x) {
 type_of <- function(x) {
   type <- typeof(x)
   if (is_formulaish(x)) {
-    if (identical(node_car(x), sym_def)) {
+    if (identical(node_car(x), colon_equals_sym)) {
       "definition"
     } else {
       "formula"
@@ -366,12 +400,25 @@ type_of <- function(x) {
 
 #' Dispatch on base types
 #'
+#' @description
+#'
+#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("experimental")}
+#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("questioning")}
+#'
 #' `switch_type()` is equivalent to
 #' \code{\link[base]{switch}(\link{type_of}(x, ...))}, while
 #' `switch_class()` switchpatches based on `class(x)`. The `coerce_`
 #' versions are intended for type conversion and provide a standard
 #' error message when conversion fails.
 #'
+#'
+#' @section Life cycle:
+#'
+#' * Like [type_of()], `switch_type()` and `coerce_type()` are
+#'   experimental functions.
+#'
+#' * `switch_class()` and `coerce_class()` are experimental functions.
+#'
 #' @param .x An object from which to dispatch.
 #' @param ... Named clauses. The names should be types as returned by
 #'   [type_of()].
@@ -382,6 +429,7 @@ type_of <- function(x) {
 #'   `.to` inherits from the S3 class `"AsIs"` (see [base::I()]).
 #' @seealso [switch_lang()]
 #' @export
+#' @keywords internal
 #' @examples
 #' switch_type(3L,
 #'   double = "foo",
@@ -449,26 +497,31 @@ coerce_class <- function(.x, .to, ...) {
   switch(class(.x), ..., abort_coercion(.x, .to))
 }
 abort_coercion <- function(x, to_type) {
-  x_type <- friendly_type(type_of(x))
+  x_type <- friendly_type_of(x)
   if (!inherits(to_type, "AsIs")) {
-    to_type <- friendly_type(to_type)
+    to_type <- as_friendly_type(to_type)
   }
   abort(paste0("Can't convert ", x_type, " to ", to_type))
 }
 
 #' Format a type for error messages
 #'
+#' @section Life cycle:
+#'
+#' * Like [type_of()], `friendly_type()` is experimental.
+#'
 #' @param type A type as returned by [type_of()] or [lang_type_of()].
 #' @return A string of the prettified type, qualified with an
 #'   indefinite article.
 #' @export
+#' @keywords internal
 #' @examples
 #' friendly_type("logical")
 #' friendly_type("integer")
 #' friendly_type("string")
 #' @export
 friendly_type <- function(type) {
-  friendly <- friendly_type_of(type)
+  friendly <- as_friendly_type(type)
   if (!is_null(friendly)) {
     return(friendly)
   }
@@ -486,7 +539,14 @@ friendly_type <- function(type) {
   type
 }
 
-friendly_type_of <- function(type) {
+friendly_type_of <- function(x) {
+  if (is.object(x)) {
+    sprintf("a `%s` object", class(x)[[1]])
+  } else {
+    as_friendly_type(typeof(x))
+  }
+}
+as_friendly_type <- function(type) {
   switch(type,
     logical = "a logical vector",
     integer = "an integer vector",
@@ -510,6 +570,7 @@ friendly_type_of <- function(type) {
     pairlist = "a pairlist node",
     expression = "an expression vector",
     quosure = "a quosure",
+    formula = "a formula",
 
     char = "an internal string",
     promise = "an internal promise",
@@ -520,7 +581,9 @@ friendly_type_of <- function(type) {
     primitive = ,
     builtin = ,
     special = "a primitive function",
-    closure = "a function"
+    closure = "a function",
+
+    type
   )
 }
 
@@ -547,10 +610,17 @@ friendly_expr_type_of <- function(type) {
 
 #' Dispatch on call type
 #'
+#' @description
+#'
+#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("experimental")}
+#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("questioning")}
+#'
 #' `switch_lang()` dispatches clauses based on the subtype of call, as
 #' determined by `lang_type_of()`. The subtypes are based on the type
 #' of call head (see details).
 #'
+#' @details
+#'
 #' Calls (objects of type `language`) do not necessarily call a named
 #' function. They can also call an anonymous function or the result of
 #' some other expression. The language subtypes are organised around
@@ -571,7 +641,7 @@ friendly_expr_type_of <- function(type) {
 #'   calls. It is generally a good idea if your function treats
 #'   `bar()` and `foo::bar()` similarly.
 #'
-#' * Finally, it is possible to have a literal (see [is_expr()] for a
+#' * Finally, it is possible to have a literal (see [is_expression()] for a
 #'   definition of literals) as call head. In most cases, this will be
 #'   a function inlined in the call (this is sometimes an expedient
 #'   way of dealing with scoping issues). For calls with a literal
@@ -582,15 +652,22 @@ friendly_expr_type_of <- function(type) {
 #'
 #' The reason we use the term _node head_ is because calls are
 #' structured as tree objects. This makes sense because the best
-#' representation for language code is a parse tree, with the tree
-#' hierarchy determined by the order of operations. See [pairlist] for
-#' more on this.
+#' representation for language code is a tree whose hierarchy is
+#' determined by the order of operations. See [node] for more on this.
+#'
+#'
+#' @section Life cycle:
+#'
+#' All these functions are in the questioning stage and likely to be
+#' removed from the package.
 #'
 #' @inheritParams switch_type
 #' @param .x,x A language object (a call). If a formula quote, the RHS
 #'   is extracted first.
 #' @param ... Named clauses. The names should be types as returned by
 #'   `lang_type_of()`.
+#'
+#' @keywords internal
 #' @export
 #' @examples
 #' # Named calls:
@@ -638,6 +715,10 @@ lang_type_of <- function(x) {
   }
 }
 
+# Exported `switch_lang()` function is likely to be deprecated in the
+# future. Use switch_call() internally:
+switch_call <- switch_lang
+
 #' Is an object copyable?
 #'
 #' When an object is modified, R generally copies it (sometimes
@@ -724,3 +805,48 @@ is_equal <- function(x, y) {
 is_reference <- function(x, y) {
   .Call(rlang_is_reference, x, y)
 }
+
+
+# Use different generic name to avoid import warnings when loading
+# packages that import all of rlang after it has been load_all'd
+rlang_type_sum <- function(x) {
+  if (is_installed("pillar")) {
+    pillar::type_sum(x)
+  } else {
+    UseMethod("type_sum")
+  }
+}
+
+type_sum.ordered <- function(x) "ord"
+type_sum.factor <- function(x) "fct"
+type_sum.POSIXct <- function(x) "dttm"
+type_sum.difftime <- function(x) "time"
+type_sum.Date <- function(x) "date"
+type_sum.data.frame <- function(x) class(x)[[1]]
+
+type_sum.default <- function(x) {
+  if (!is.object(x)) {
+    switch(typeof(x),
+      logical = "lgl",
+      integer = "int",
+      double = "dbl",
+      character = "chr",
+      complex = "cpl",
+      builtin = ,
+      special = ,
+      closure = "fn",
+      environment = "env",
+      symbol =
+        if (is_missing(x)) {
+          "missing"
+        } else {
+          "sym"
+        },
+      typeof(x)
+    )
+  } else if (!isS4(x)) {
+    paste0("S3: ", class(x)[[1]])
+  } else {
+    paste0("S4: ", methods::is(x)[[1]])
+  }
+}
