@@ -16,18 +16,21 @@ enum dots_expansion_op {
   OP_EXPR_UQS,
   OP_EXPR_UQN,
   OP_EXPR_FIXUP,
+  OP_EXPR_DOT_DATA,
   OP_QUO_NONE,
   OP_QUO_UQ,
   OP_QUO_UQE,
   OP_QUO_UQS,
   OP_QUO_UQN,
   OP_QUO_FIXUP,
+  OP_QUO_DOT_DATA,
   OP_VALUE_NONE,
   OP_VALUE_UQ,
   OP_VALUE_UQE,
   OP_VALUE_UQS,
   OP_VALUE_UQN,
   OP_VALUE_FIXUP,
+  OP_VALUE_DOT_DATA,
   OP_DOTS_MAX
 };
 
