@@ -13,7 +13,7 @@ sexp* new_preserved_empty_list() {
   return empty_list;
 }
 
-void signal_soft_deprecation(const char* msg) {
+void signal_soft_deprecated(const char* msg) {
   sexp* opt = r_peek_option("lifecycle_force_verbose_retirement");
   if (r_is_true(opt)) {
     r_warn(msg);
